import json
import os
import unittest
from unittest.mock import patch

import pytest

from release_exporter.exceptions import FileExists, InvalidToken
from release_exporter.requests import GitHubRequest, GitLabRequest


# ---------------------------------- GitHub -----------------------------------------


def test_GitHubRequest_fail():
    with pytest.raises(FileExists,
                       message='CHANGELOG.md already exists at ' + os.getcwd() + '. Use --force to override.'):
        a = GitHubRequest()


def test_GitHubRequest_fail_2():
    with pytest.raises(InvalidToken,
                       message="Oops! GitHub requires you to generate a private token to get the details. See "
                               "https://help.github.com/articles/creating-a-personal-access-token-for-the-command-line/ "
                               "for more information."):
        a = GitHubRequest(force=True)


class TestGitHubRequestInit(unittest.TestCase):

    def setUp(self):
        self.github_request_class = GitHubRequest(force=True, token='hello')

    def test_init(self):
        self.assertEqual(self.github_request_class.request_header, None)
        self.assertEqual(self.github_request_class.api_url, 'https://api.github.com/graphql')
        self.assertEqual(str(type(self.github_request_class.info)), "<class 'giturlparse.parser.Parsed'>")

    def test_init_condition(self):
        local_car = GitHubRequest(force=True, token='hello',
                                  repo_url='https://github.com/akshaybabloo/release-exporter')
        self.assertEqual(local_car.info.name, 'release-exporter')


class TestGitHubRequestFail(unittest.TestCase):

    def setUp(self):
        self.github_request_class = GitHubRequest(force=True, token='hello')

    def test_total_number_releases(self):
        with pytest.raises(KeyError, message='Wrong credentials given. Please check if you have the correct token.'):
            response = self.github_request_class._total_number_releases()

    def test_releases(self):
        with pytest.raises(KeyError, message='Wrong credentials given. Please check if you have the correct token.'):
            response = self.github_request_class.releases()


class TestGitHubRequest(unittest.TestCase):

    def setUp(self):
        self.github_request_class = GitHubRequest(force=True, token=os.environ['GITHUB_TOKEN'],
                                                  repo_url='https://github.com/akshaybabloo/release-exporter')

    def test_total_number_releases(self):
        response = self.github_request_class._total_number_releases()

        self.assertIsInstance(response, int)

    def test_release(self):
        response = self.github_request_class.releases()
        self.assertIn('tag', json.dumps(response))


# ---------------------------------- GitLab -----------------------------------------


def test_GitLabRequest_fail():
    with pytest.raises(FileExists,
                       message='CHANGELOG.md already exists at ' + os.getcwd() + '. Use --force to override.'):
        a = GitLabRequest()


def test_GitLabRequest_fail_2():
    with pytest.raises(InvalidToken,
                       message="Oops! GitLab requires you to generate a private token to get the details. See "
                               "https://docs.gitlab.com/ce/user/profile/personal_access_tokens.html "
                               "for more information."):
        a = GitLabRequest(force=True)


class TestGitLabRequestInit(unittest.TestCase):

    def setUp(self):
        self.gitlab_request_class = GitLabRequest(force=True, token='hello')

    def test_init(self):
        self.assertEqual(self.gitlab_request_class.request_header, None)
        self.assertEqual(self.gitlab_request_class.api_url, 'https://gitlab.com/api/v4/')
        self.assertEqual(str(type(self.gitlab_request_class.info)), "<class 'giturlparse.parser.Parsed'>")

    def test_init_condition(self):
        local_car = GitHubRequest(force=True, token='hello',
                                  repo_url='https://gitlab.com/akshaybabloo/test-releases')
        self.assertEqual(local_car.info.name, 'test-releases')


class TestGitLabRequestFail(unittest.TestCase):

    def setUp(self):
        self.gitlab_request_class = GitLabRequest(force=True, token='hello')

    def test_repo_id(self):
        with pytest.raises(KeyError, message='Wrong credentials given. Please check if you have the correct token.'):
            response = self.gitlab_request_class._repo_id()

    def test_releases(self):
        with pytest.raises(KeyError, message='Wrong credentials given. Please check if you have the correct token.'):
            response = self.gitlab_request_class.releases()


class TestGitLabRequest(unittest.TestCase):

    def setUp(self):
        self.gitlab_request_class = GitLabRequest(force=True, token=os.environ['GITLAB_TOKEN'],
                                                  repo_url='https://gitlab.com/akshaybabloo/test-releases')

    def test_repo_id(self):
        response = self.gitlab_request_class._repo_id()

        self.assertIsInstance(response, int)

    def test_release(self):
        response = self.gitlab_request_class.releases()
        self.assertIn('tag_name', json.dumps(response))


def test_GitLab_repo_id():
    gitlab_request_class = GitLabRequest(force=True, token=os.environ['GITLAB_TOKEN'],
                                         repo_url='https://gitlab.com/akshaybabloo/public')

    with patch('builtins.input', return_value='12345'):
        assert gitlab_request_class._repo_id() == '12345'



import os

import click

from release_exporter.exceptions import UnknownRepo
from release_exporter.formatter import github
from release_exporter.formatter import gitlab
from release_exporter.utils import get_repo_url_info
from release_exporter.version import __version__


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__)
    ctx.exit()


@click.group()
@click.option('--token', help='Token number if its a private repository.', default=None)
@click.option('--url',
              help="URL of your repository. This is optional if your current directory has .git folder with remote url.",
              default=None)
@click.option('--location', help='Local location of your repository.', default=os.getcwd())
@click.option('--version', is_flag=True, callback=print_version, expose_value=False, is_eager=True)
@click.pass_context
def cli(ctx, token, url, location):
    ctx.obj['token'] = token
    ctx.obj['repo_url'] = url
    ctx.obj['location'] = location


# TODO: Implement init as a go to token area.
# @cli.command(help='Creates .rex file.')
# def init():
#     pass


@cli.command(help='Creates markdown file.')
@click.pass_context
def markdown(ctx):
    if os.name == 'nt':
        ctx.obj['location'] = r'{}'.format(ctx.obj['location'])

    if "github" in get_repo_url_info(location=ctx.obj['location'], repo_url=ctx.obj['repo_url']).resource:
        print('GitHub detected. \n')

        github(force=True, token=ctx.obj['token'], location=ctx.obj['location'], repo_url=ctx.obj['repo_url'],
               file_type='markdown').write_markdown()

    elif "gitlab" in get_repo_url_info(location=ctx.obj['location'], repo_url=ctx.obj['repo_url']).resource:
        print('GitLab detected. \n')

        gitlab(force=True, token=ctx.obj['token'], location=ctx.obj['location'], repo_url=ctx.obj['repo_url'],
               file_type='markdown').write_markdown()

    else:
        raise UnknownRepo("Sorry, couldn't find the repository. Trying giving the repository URL by adding --url flag.")


@cli.command(help='Creates JSON file.')
@click.pass_context
def json(ctx):
    if os.name == 'nt':
        ctx.obj['location'] = r'{}'.format(ctx.obj['location'])

    if "github" in get_repo_url_info(location=ctx.obj['location'], repo_url=ctx.obj['repo_url']).resource:
        click.echo('GitHub detected. \n')

        github(force=True, token=ctx.obj['token'], location=ctx.obj['location'], repo_url=ctx.obj['repo_url'],
               file_type='json').write_json()

    elif "gitlab" in get_repo_url_info(location=ctx.obj['location'], repo_url=ctx.obj['repo_url']).resource:
        click.echo('GitLab detected. \n')

        gitlab(force=True, token=ctx.obj['token'], location=ctx.obj['location'], repo_url=ctx.obj['repo_url'],
               file_type='json').write_json()

    else:
        raise UnknownRepo("Sorry, couldn't find the repository. Trying giving the repository URL by adding --url flag.")


@cli.command(help='Creates reStructuredText file. Coming soon.')
@click.pass_context
def rest(ctx):
    # if os.name == 'nt':
    #     ctx.obj['location'] = r'{}'.format(ctx.obj['location'])
    #
    # if "github" in get_repo_url_info(location=ctx.obj['location'], repo_url=ctx.obj['repo_url']).resource:
    #     click.echo('GitHub detected. \n')
    #
    #     github(force=True, token=ctx.obj['token'], location=ctx.obj['location'], repo_url=ctx.obj['repo_url'],
    #            file_type='rest').write_json()
    #
    # elif "gitlab" in get_repo_url_info(location=ctx.obj['location'], repo_url=ctx.obj['repo_url']).resource:
    #     click.echo('GitLab detected. \n')
    #
    #     gitlab(force=True, token=ctx.obj['token'], location=ctx.obj['location'], repo_url=ctx.obj['repo_url'],
    #            file_type='rest').write_json()
    #
    # else:
    #     raise UnknownRepo("Sorry, couldn't find the repository. Trying giving the repository URL by adding --url flag.")
    raise NotImplementedError("Coming soon.")


def main():
    cli(obj={})



import configparser
import os
import shutil
import tempfile
import time
from pathlib import Path

import pytest
from giturlparse import parse

from release_exporter.exceptions import ParserError
from release_exporter.utils import get_repo_url_info, date_convert, multi_key_gitlab, description

DUP_SECTION = """\
[branch "v3"]
[travis]
    slug = akshaybabloo/gollahalli-me
[branch "v3"]
    remote = origin
    merge = refs/heads/v3
"""

PASS_SECTION = """\
[remote "origin"]
    url = git@github.com:akshaybabloo/release-exporter.git
    fetch = +refs/heads/*:refs/remotes/origin/*
[branch "master"]
    remote = origin
    merge = refs/heads/master
"""


def temp_file(value):
    temp_location = tempfile.gettempdir()
    os.chdir(temp_location)

    p = Path('.git/')

    if p.is_dir():
        shutil.rmtree(temp_location + os.sep + '.git')

    time.sleep(1)

    os.mkdir('.git/')

    with open(temp_location + os.sep + '.git' + os.sep + 'config', 'w') as config_file:
        config_file.write(value)

# ---------------------- get_repo_url_info ------------------


def test_get_repo_url_info_args():
    content = get_repo_url_info(
        repo_url='https://github.com/akshaybabloo/release-exporter.git')

    assert parse(
        'https://github.com/akshaybabloo/release-exporter.git').owner == content.owner


def test_get_repo_url_info_fail():
    with pytest.raises(ParserError,
                       message='Git config file does not exist please provide the repository url by using --url.'):
        content = get_repo_url_info(location='yo')


def test_get_repo_url_info_pass():
    t = tempfile.gettempdir()
    temp_file(PASS_SECTION)

    content = get_repo_url_info(location=t)

    assert content.owner == 'akshaybabloo'


def test_get_repo_url_info_fail_2():
    t = tempfile.gettempdir()
    temp_file(DUP_SECTION)

    with pytest.raises(configparser.DuplicateSectionError,
                       message='There seems to be a duplicate section in your config. Try giving the repository URL by using --url.'):
        content = get_repo_url_info(location=t)


# ---------------- date_convert -----------------------


def test_date_convert_fail():
    with pytest.raises(ValueError, message="Unknown string format"):
        content = date_convert('10-10-2019T12:12:12z')


def test_date_convert_pass():
    content = date_convert('10-10-2019')
    assert content == '2019-10-10'


def test_date_convert_pass2():
    content = date_convert('2008-01-14T04:33:35Z')
    assert content == '2008-01-14'


# ----------------- multi_key_gitlab ------------------


def test_multi_key_gitlab_pass():
    data = {
        "owner": {
            "username": "user"
        }
    }

    content = multi_key_gitlab(data)
    assert content == "user"


def test_multi_key_gitlab_pass_except():
    data = {
        "owner": "user"
    }

    content = multi_key_gitlab(data)
    assert content == None


# ------------- description ------------

def test_description_pass():
    provider = "some provider"
    repo_name = "some repo name"
    tags_number = 22

    expected = '\n'.join(['+-----------------+----------------+',
                          '| Provider        | some provider  |',
                          '+-----------------+----------------+',
                          '| Repository Name | some repo name |',
                          '+-----------------+----------------+',
                          '| Number of Tags  | 22             |',
                          '+-----------------+----------------+'])

    actual = description(provider, repo_name, tags_number)

    assert actual == expected


def test_description_fail():
    provider = "some provider"
    repo_name = "some repo name"

    expected = '\n'.join(['+-----------------+----------------+',
                          '| Provider        | some provider  |',
                          '+-----------------+----------------+',
                          '| Repository Name | some repo name |',
                          '+-----------------+----------------+',
                          '| Number of Tags  | 22             |',
                          '+-----------------+----------------+'])

    actual = description(provider, repo_name)

    assert actual != expected



import io
import os
import unittest
from unittest.mock import patch

from release_exporter.formatter import github, gitlab


# ------------------------- GitHub --------------------------


class TestGitHubFormatMarkdown(unittest.TestCase):

    def setUp(self):
        self.github_format = github(force=True, token=os.environ['GITHUB_TOKEN'],
                                    repo_url='https://github.com/akshaybabloo/release-exporter')

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_1(self, n, expected_output, mock_stdout):
        request = self.github_format._converter()
        self.assertIn(expected_output, mock_stdout.getvalue())

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_2(self, n, expected_output, mock_stdout):
        request = self.github_format.write_markdown()
        self.assertIn(expected_output, mock_stdout.getvalue())

    def test_convert(self):
        request = self.github_format._converter()

        self.assertIsInstance(request, tuple)
        self.assertIn('changelog', request[0])

    def test_output(self):
        self.assert_stdout_1('', 'Provider')

    def test_write_markdown(self):
        self.assert_stdout_2('', 'Done')


class TestGitHubFormatJson(unittest.TestCase):

    def setUp(self):
        self.github_format = github(force=True, token=os.environ['GITHUB_TOKEN'],
                                    repo_url='https://github.com/akshaybabloo/release-exporter', file_type='json')

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_1(self, n, expected_output, mock_stdout):
        request = self.github_format._converter()
        self.assertIn(expected_output, mock_stdout.getvalue())

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_2(self, n, expected_output, mock_stdout):
        request = self.github_format.write_json()
        self.assertIn(expected_output, mock_stdout.getvalue())

    def test_convert(self):
        request = self.github_format._converter()

        self.assertIs(request, None)
        self.assertIn('provider', self.github_format._dict_repo_template())

    def test_output(self):
        self.assert_stdout_1('', 'Provider')

    def test_write_markdown(self):
        self.assert_stdout_2('', 'Done')


# ------------------------- GitLab --------------------------


class TestGitLabFormatMarkdown(unittest.TestCase):

    def setUp(self):
        self.gitlab_format = gitlab(force=True, token=os.environ['GITLAB_TOKEN'],
                                    repo_url='https://gitlab.com/akshaybabloo/test-releases')

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_1(self, n, expected_output, mock_stdout):
        request = self.gitlab_format._converter()
        self.assertIn(expected_output, mock_stdout.getvalue())

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_2(self, n, expected_output, mock_stdout):
        request = self.gitlab_format.write_markdown()
        self.assertIn(expected_output, mock_stdout.getvalue())

    def test_convert(self):
        request = self.gitlab_format._converter()

        self.assertIsInstance(request, tuple)
        self.assertIn('changelog', request[0])

    def test_output(self):
        self.assert_stdout_1('', 'Provider')

    def test_write_markdown(self):
        self.assert_stdout_2('', 'Done')


class TestGitLabFormatJson(unittest.TestCase):

    def setUp(self):
        self.gitlab_format = gitlab(force=True, token=os.environ['GITLAB_TOKEN'],
                                    repo_url='https://gitlab.com/akshaybabloo/test-releases', file_type='json')

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_1(self, n, expected_output, mock_stdout):
        request = self.gitlab_format._converter()
        self.assertIn(expected_output, mock_stdout.getvalue())

    @patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_2(self, n, expected_output, mock_stdout):
        request = self.gitlab_format.write_json()
        self.assertIn(expected_output, mock_stdout.getvalue())

    def test_convert(self):
        request = self.gitlab_format._converter()

        self.assertIs(request, None)
        self.assertIn('provider', self.gitlab_format._dict_repo_template())

    def test_output(self):
        self.assert_stdout_1('', 'Provider')

    def test_write_markdown(self):
        self.assert_stdout_2('', 'Done')



import os
import unittest
from collections import namedtuple

import pytest

from release_exporter.base import FormatBase, FILE_TYPE
from release_exporter.exceptions import FileExists


class TestFormatBase(unittest.TestCase):

    def setUp(self):
        self.format_base = FormatBase(force=True)

    def test_args(self):
        self.assertEqual(self.format_base.location, os.getcwd())
        self.assertEqual(self.format_base.file_name, 'CHANGELOG')
        self.assertTrue(self.format_base.force)
        self.assertEqual(self.format_base.file_type, 'markdown')
        self.assertEqual(self.format_base.all_content, [])
        self.assertIsNone(self.format_base.tag_name)
        self.assertIsNone(self.format_base.date)
        self.assertIsNone(self.format_base.description)
        self.assertIsNone(self.format_base.compare_url)
        self.assertIsNone(self.format_base.total_number_tags)
        self.assertIsNone(self.format_base.iter_count)
        self.assertEqual(self.format_base.list_descriptions, [])

    def test_get_file_ext(self):
        self.assertEqual(self.format_base._get_file_ext(), '.md')

    def test_write_markdown(self):
        with pytest.raises(NotImplementedError):
            self.format_base.write_markdown()

    def test_write_json(self):
        with pytest.raises(NotImplementedError):
            self.format_base.write_json()

    def test_header(self):
        actual = """\
# Changelog

All notable changes to this project will be documented in this file.

The format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)
and this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).

## [Unreleased]

"""
        self.assertEqual(self.format_base._header(), actual)

    def test_converter(self):
        self.assertIsNone(self.format_base._converter())

    def test_total_number_releases(self):
        self.assertIs(self.format_base._total_number_releases(), None)

    def test_releases(self):
        self.assertIs(self.format_base.releases(), None)


class FormatRequestBaseBody(unittest.TestCase):
    def setUp(self):
        self.format_base = FormatBase(force=True)
        self.format_base.iter_count = 0
        self.format_base.total_number_tags = 2
        self.format_base.tag_name = 'test'
        self.format_base.date = '2008-10-10'
        self.format_base.description = 'hello'

    def test_body_less_condition(self):
        expected = '\n'.join(['## [test] - 2008-10-10',
                              '',
                              'hello'])

        self.assertIsInstance(self.format_base._body(), str)

    def test_body_else_condition(self):
        self.format_base.iter_count = 30

        expected = '\n'.join(['## test - 2008-10-10',
                              '',
                              'hello',
                              ''])
        self.assertIsInstance(self.format_base._body(), str)

    def test_footer(self):
        self.format_base.tag_name = '1'
        self.format_base.repo_url = "http://"

        expected = '[1]: http://\n'
        self.assertEqual(self.format_base._footer(), expected)

    def test__dict_data_template(self):
        expected = {
            "tagName": '1',
            "description": 'hello',
            "createdAt": '2008-10-10',
            "compareUrl": 'http://'
        }

        self.assertEqual(self.format_base._dict_data_template('1', 'hello', '2008-10-10', 'http://'), expected)

    def test_dict_repo_template(self):
        local_nt = namedtuple('local_nt', 'name resource owner')
        k = local_nt(name='akshay', resource='some', owner='akshay')
        self.format_base.info = k
        self.format_base.total_number_tags = 10
        self.format_base.list_descriptions = []

        expected = {
            'name': "akshay",
            'resource': 'some resource',
            'owner': 'akshay',
            'repoUrl': 'https://some/akshay/akshay',
            'totalTags': 1,
            "data": []
        }

        self.assertNotEqual(self.format_base._dict_repo_template(), expected)


def test_file_type_constant():
    actual = {
        'markdown': '.md',
        'json': '.json',
        'rest': '.rst'
    }
    assert FILE_TYPE == actual


def test_format_base_fail():
    with pytest.raises(FileExists):
        format_base = FormatBase()


def test_format_base_file_type_fail():
    format_base = FormatBase(force=True, file_type='hello')
    assert format_base._get_file_ext() is None



import json

from release_exporter.requests import GitHubRequest, GitLabRequest
from release_exporter.utils import date_convert, description


class GitHubFormat(GitHubRequest):
    """
    Changelog of GitHub.
    """

    def __init__(self, *args, **kwargs):
        super(GitHubFormat, self).__init__(*args, **kwargs)

        self.compare_url = 'https://' + self.info.resource + '/' + self.info.owner + '/' + self.info.name + '/compare/'

    def write_json(self):
        """
        Writes down a CHANGELOG.json file.
        """

        self._converter()

        with open(self.file_name + '.' + self.file_type, 'w') as json_file:
            json.dump(self._dict_repo_template(), json_file, indent=4)

        print('\n' + 'Done ' + u"\U0001F44D")

    def write_markdown(self):
        """
        Writes down a CHANGELOG.md file.
        """
        with open('CHANGELOG.md', 'w') as md_file:
            md_file.writelines(self._converter())

        print('\n' + 'Done ' + u"\U0001F44D")

    def _converter(self):
        """
        A tuple of formatted tag name, description, created at and the compare links.

        Returns
        -------
        tuple: tuple
            A tuple of list.
        """

        temp = self.releases()['data']['repository']['releases']['edges']
        temp_l = []

        self.total_number_tags = sum(1 for k in temp if k['node']['tag']['name'])

        print(description(provider=self.info.resource, repo_name=self.info.name, tags_number=self.total_number_tags))

        if self.file_type == 'markdown':

            self.all_content.append(self._header())

            for count, edge in enumerate(temp):
                self.iter_count = count
                temp_l.append(edge['node']['tag']['name'])
                self.tag_name = edge['node']['tag']['name']
                self.description = edge['node']['description'].replace('\r\n', '\n')
                self.date = date_convert(edge['node']['createdAt'])
                self.all_content.append(self._body())

            pair = list(['{}...{}'.format(a, b) for a, b in zip(temp_l, ['HEAD'] + temp_l[:-1])])

            for count, tags in enumerate(pair):
                if count < 1:
                    self.all_content.append('[Unreleased]: ' + self.compare_url + tags + '\n')
                else:
                    self.all_content.append('[' + tags.split('...')[1] + ']: ' + self.compare_url + tags + '\n')

            return tuple(self.all_content)

        elif self.file_type == 'json':

            temp_l2 = []
            tag_comp_url_temp = []
            self.list_descriptions.insert(0, self._dict_data_template(tag_name='Unreleased', description='', date=''))

            for count, edge in enumerate(temp):
                self.iter_count = count
                temp_l2.append(edge['node']['tag']['name'])

                self.list_descriptions.append(self._dict_data_template(tag_name=edge['node']['tag']['name'],
                                                                       description=edge['node'][
                                                                           'description'].replace('\r\n',
                                                                                                  '\n'),
                                                                       date=date_convert(
                                                                           edge['node']['createdAt'])))

            pair = list(['{}...{}'.format(a, b) for a, b in zip(temp_l2, ['HEAD'] + temp_l2[:-1])])

            for tags in pair:
                tag_comp_url_temp.append(self.compare_url + tags)

            for count, urls in enumerate(tag_comp_url_temp):
                self.list_descriptions[count]['compareUrl'] = urls


github = GitHubFormat


class GitLabFormat(GitLabRequest):
    """
    Changelog of GitLab.
    """

    def __init__(self, *args, **kwargs):
        super(GitLabFormat, self).__init__(*args, **kwargs)

        self.compare_url = 'https://' + self.info.resource + '/' + self.info.owner + '/' + self.info.name + '/compare/'

    def write_json(self):
        self._converter()

        with open(self.file_name + '.' + self.file_type, 'w') as json_file:
            json.dump(self._dict_repo_template(), json_file, indent=4)

        print('\n' + 'Done ' + u"\U0001F44D")

    def write_markdown(self):
        """
        Writes down a CHANGELOG.md file.
        """
        with open('CHANGELOG.md', 'w') as md_file:
            md_file.writelines(self._converter())

        print('\n' + 'Done ' + u"\U0001F44D")

    def _converter(self):
        """
        A tuple of formatted tag name, description, created at and the compare links.

        Returns
        -------
        tuple: tuple
            A tuple of list.
        """

        temp = self.releases()
        temp_l = []

        self.total_number_tags = sum(1 for k in temp if k['name'])

        print(description(provider=self.info.resource, repo_name=self.info.name, tags_number=self.total_number_tags))

        if self.file_type == 'markdown':

            self.all_content.append(self._header())

            for count, content in enumerate(temp):
                self.iter_count = count
                temp_l.append(content['name'])
                self.tag_name = content['name']
                self.description = content['release']['description'].replace('\r\n', '\n')
                self.date = date_convert(content['commit']['created_at'])
                self.all_content.append(self._body())

            pair = list(['{}...{}'.format(a, b) for a, b in zip(temp_l, ['HEAD'] + temp_l[:-1])])

            self.all_content.append('\n')

            for count, tags in enumerate(pair):
                if count < 1:
                    self.all_content.append('[Unreleased]: ' + self.compare_url + tags + '\n')
                else:
                    self.all_content.append('[' + tags.split('...')[1] + ']: ' + self.compare_url + tags + '\n')

            return tuple(self.all_content)

        elif self.file_type == 'json':

            temp_l2 = []
            tag_comp_url_temp = []
            self.list_descriptions.insert(0, self._dict_data_template(tag_name='Unreleased', description='', date=''))

            for count, content in enumerate(temp):
                self.iter_count = count
                temp_l2.append(content['name'])

                self.list_descriptions.append(self._dict_data_template(tag_name=content['name'],
                                                                       description=content['release'][
                                                                           'description'].replace('\r\n', '\n'),
                                                                       date=date_convert(
                                                                           content['commit']['created_at'])))

            pair = list(['{}...{}'.format(a, b) for a, b in zip(temp_l2, ['HEAD'] + temp_l2[:-1])])

            for tags in pair:
                tag_comp_url_temp.append(self.compare_url + tags)

            for count, urls in enumerate(tag_comp_url_temp):
                    self.list_descriptions[count]['compareUrl'] = urls


gitlab = GitLabFormat



import os
import shutil
import tempfile
import time
from pathlib import Path
from unittest.mock import patch

from click.testing import CliRunner

from release_exporter.cli import cli
from release_exporter.exceptions import UnknownRepo


class Values:

    def __init__(self):
        self.token = 123
        self.url = 123
        self.location = 123
        self.version = 123

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, value):
        setattr(self, key, value)


SECTION = """\
[remote "origin"]
    url = git@gollahalli.com:akshaybabloo/release-exporter.git
    fetch = +refs/heads/*:refs/remotes/origin/*
[branch "master"]
    remote = origin
    merge = refs/heads/master
"""

GITLAB_SECTION = """\
[remote "origin"]
    url = git@gitlab.com:akshaybabloo/release-exporter.git
    fetch = +refs/heads/*:refs/remotes/origin/*
[branch "master"]
    remote = origin
    merge = refs/heads/master
"""


def temp_file(value):
    temp_location = tempfile.gettempdir()
    os.chdir(temp_location)

    p = Path('.git/')

    if p.is_dir():
        shutil.rmtree(temp_location + os.sep + '.git')

    time.sleep(1)

    os.mkdir('.git/')

    with open(temp_location + os.sep + '.git' + os.sep + 'config', 'w') as config_file:
        config_file.write(value)


def test_cli_help():
    runner = CliRunner()
    result = runner.invoke(cli, ['--help'])

    assert result.exit_code == 0
    assert 'help' in result.output


def test_print_version():
    runner = CliRunner()
    result = runner.invoke(cli, ['--version'])

    assert result.exit_code == 0
    assert isinstance(result.output, str)


def test_markdown_fail():
    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', 'markdown'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, KeyError)


def test_json_fail():
    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', 'json'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, KeyError)


@patch('os.name', 'nt')
def test_markdown_fail_2():
    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', '--location', os.getcwd(), 'markdown'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, KeyError)


@patch('os.name', 'nt')
def test_json_fail_2():
    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', '--location', os.getcwd(), 'json'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, KeyError)


def test_markdown_exception():
    t = tempfile.gettempdir()
    temp_file(SECTION)

    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', '--location', t, 'markdown'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, UnknownRepo)


def test_json_fail_exception():
    t = tempfile.gettempdir()
    temp_file(SECTION)

    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', '--location', t, 'json'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, UnknownRepo)


def test_markdown_fail_3():
    t1 = tempfile.gettempdir()
    temp_file(GITLAB_SECTION)

    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', '--location', t1, 'markdown'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, KeyError)
    assert 'GitLab' in result.output


def test_json_fail_3():
    t1 = tempfile.gettempdir()
    temp_file(GITLAB_SECTION)

    runner = CliRunner()
    result = runner.invoke(cli, ['--token', 'some_token', '--location', t1, 'json'], obj=Values())

    assert result.exit_code == -1
    assert isinstance(result.exception, KeyError)
    assert 'GitLab' in result.output



import codecs
import os
from setuptools import setup

from release_exporter.version import __version__

here = os.path.abspath(os.path.dirname(__file__)) + os.sep


def get_requirements(*parts):
    return codecs.open(os.path.join(here, *parts), 'r').read().splitlines()


setup(
    name='release-exporter',
    version=__version__,
    install_requires=get_requirements('requirements.txt'),
    packages=['tests', 'release_exporter'],
    url='https://blog.gollahalli.com/export-your-github-and-gitlab-releases-as-a-changelog/',
    license='MIT',
    author='Akshay Raj Gollahalli',
    author_email='akshay@gollahalli.com',
    description='Release exporter for GitHub and GitLab.',
    long_description=open('README.rst').read(),
    keywords=['changelog', 'releases'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Intended Audience :: Developers',
        'Environment :: Console',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Utilities'
    ],
    entry_points={
        'console_scripts': [
            'rex = release_exporter.cli:main'
        ]
    }
)



import json

import requests

from release_exporter.base import FormatBase
from release_exporter.exceptions import InvalidToken
from release_exporter.utils import get_repo_url_info, multi_key_gitlab


class GitHubRequest(FormatBase):
    """
    GitHub request base.
    """

    def __init__(self, *args, **kwargs):
        super(GitHubRequest, self).__init__(*args, **kwargs)
        self.request_headers = {'Authorization': 'token %s' % self.token}
        self.api_url = 'https://api.github.com/graphql'

        if self.token is None:
            raise InvalidToken(
                "Oops! GitHub requires you to generate a private token to get the details. See "
                "https://help.github.com/articles/creating-a-personal-access-token-for-the-command-line/ "
                "for more information.")

        if self.repo_url is not None:
            self.info = get_repo_url_info(self.location, repo_url=self.repo_url)
        else:
            self.info = get_repo_url_info(self.location)

    def _total_number_releases(self):
        """
        Queries total number of releases.

        Returns
        -------
        int: int
            Number of releases.
        """
        _json = {"query": """
            {
              repository(owner: """ + """\"{}\",""".format(self.info.owner) + """ name: """ + """\"{}\")""".format(
            self.info.name) + """ {
                releases {
                  totalCount
                }
              }
            }
        """}

        r = requests.post(url=self.api_url, json=_json, headers=self.request_headers)
        try:
            return int(json.loads(r.text)['data']['repository']['releases']['totalCount'])
        except KeyError:
            raise KeyError('Wrong credentials given. Please check if you have the correct token.')

    def releases(self):
        """
        A JSON object with name of the repository, tag name, description and the created date and time.

        Returns
        -------
        JSON: dict
            A JSON object.
        """
        _json = {
            "query": """
                query {""" +
                     """repository(owner: \"{}\", name: \"{}\") """.format(self.info.owner, self.info.name) + """{
                    releases(""" + """first:{}""".format(self._total_number_releases()) + """, orderBy: {field: CREATED_AT, direction: DESC}){
                      edges{
                        node{
                          name
                          tag{
                            name
                          }
                          description
                          createdAt
                        }
                      }
                    }
                  }
                }
            """
        }

        r = requests.post(url=self.api_url, json=_json, headers=self.request_headers)
        return json.loads(r.text)


class GitLabRequest(FormatBase):
    """
    GitLab request base.
    """

    def __init__(self, *args, **kwargs):
        super(GitLabRequest, self).__init__(*args, **kwargs)
        self.request_headers = {'Private-Token': '%s' % self.token}
        self.api_url = 'https://gitlab.com/api/v4/'

        if self.token is None:
            raise InvalidToken(
                "Oops! GitLab requires you to generate a private token to get the details. See "
                "https://docs.gitlab.com/ce/user/profile/personal_access_tokens.html "
                "for more information.")

        if self.repo_url is not None:
            self.info = get_repo_url_info(self.location, repo_url=self.repo_url)
        else:
            self.info = get_repo_url_info(self.location)

    def _repo_id(self):
        """
        Searches and returns the repository ID based on the repository name. If the repository is not found then a table
        of repository is shown so that the user can manually enter the ID of their repository.

        Returns
        -------
        id_number: int
            Repository ID.

        """
        url = self.api_url + 'projects?search={}'.format(self.info.name)

        r = requests.get(url=url, headers=self.request_headers)
        id_number = json.loads(r.text)

        if len(id_number) > 1:
            print(
                "The search resulted in more that one repository. Please check your repository name and type in it's ID")
            print('ID - Repository Name - Username')

            for content in id_number:
                print('{id} - {repo_name} - {user_name}'.format(id=content['id'], repo_name=content['name'],
                                                                user_name=multi_key_gitlab(content)))

            id_number = input('ID > ')
            return id_number

        try:
            return id_number[0]['id']
        except KeyError:
            raise KeyError('Wrong credentials given. Please check if you have the correct token.')

    def releases(self):
        """
        A JSON object containing name of the repository, tag name, description and the created date and time

        Returns
        -------
        JSON: dict
            A dict object.
        """
        url = 'https://gitlab.com/api/v4/projects/{id}/repository/tags'.format(id=self._repo_id())

        r = requests.get(url=url, headers=self.request_headers)
        return json.loads(r.text)



import configparser
import datetime
import os

import dateutil.parser
from giturlparse import parse
from tabulate import tabulate

from .exceptions import ParserError


def get_repo_url_info(location=os.getcwd(), repo_url=None):
    """
    Returns the parsed URL.

    Parameters
    ----------
    location: str
        Absolute location of the current directory.
    repo_url: str
        URL of the repository.

    Returns
    -------
    parse: giturlparse.parser.Parsed
        A named tuple.

    """
    try:
        if repo_url is None:
            config = configparser.ConfigParser()
            config.read(location + os.sep + '.git' + os.sep + 'config')
            if 'remote "origin"' in config.sections():
                return parse(config['remote "origin"']['url'])
            else:
                raise ParserError('Git config file does not exist please provide the repository url by using --url.')
        else:
            return parse(repo_url + '.git')
    except configparser.DuplicateSectionError:
        raise configparser.DuplicateSectionError(
            'There seems to be a duplicate section in your config. Try giving the repository URL by using --url.')


def date_convert(date):
    """
    Converts ISO8601 date and time and returns only the date.

    Parameters
    ----------
    date: str
        datetime string.

    Returns
    -------
    date: str
        Date as Y-m-d format..

    """
    try:
        date = datetime.datetime.strptime(date, '%Y-%m-%dT%H:%M:%Sz')
        date = date.strftime('%Y-%m-%d')
    except ValueError:
        date = dateutil.parser.parse(date).date().strftime('%Y-%m-%d')
    return date


def multi_key_gitlab(value):
    """
    Returns the username, if an exception occurs None is returned.

    Parameters
    ----------
    value: dict
        A dictionary of GitLab.

    Returns
    -------
    value: str or None
        Username or none.
    """

    try:
        return value['owner']['username']
    except (KeyError, TypeError):
        return None


def description(provider=None, repo_name=None, tags_number=None):
    """
    Description generator.

    Parameters
    ----------
    provider: str
        Name of the Git host.
    repo_name: str
        Repository name.
    tags_number: str or int
        Number of tags.
    
    Return
    ------
    tabulate: str
        A tabulated structure of the input.
    """
    table = [
        ['Provider', provider],
        ['Repository Name', repo_name],
        ['Number of Tags', tags_number]
    ]

    return tabulate(table, tablefmt="grid")
